<?php

namespace Drupal\swapcard_content\Plugin\QueueWorker;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\media\MediaInterface;

/**
 * Creates QueueWorker for deleting over Swapcard ecosystem.
 *
 * @QueueWorker(
 *   id = "swapcard_content_queue_swapcard_delete",
 *   title = @Translation("Swapcard Delete entities"),
 *   cron = {"time" = 90}
 * )
 */
class SwapcardQueueWorkerDelete extends SwapcardQueueWorkerSessions {

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {

    if (empty($data) || !isset($data['entity'])) {
      return;
    }

    if ($data['entity'] instanceof ContentEntityInterface) {

      if (!$this->config->get('dry_run') && $this->config->get('purge')) {

        // Delete this entity.
        $data['entity']->delete();
        // And notify (in shell only).
        $this->notify($data['entity'], 'Deleted');

        // A special case for Session <> Speaker.
        $entities = $this->checkEventEntities();
        if (isset($entities['speakers_key']) && $data['entity']->bundle() == 'swapcard_session' && isset($data['data'])) {
          $this->orphanSpeakers($data['data'], $data['entity']);
        }

        if (!$data['entity'] instanceof MediaInterface) {

          // Swapcard content media plugin handling.
          // Take care of some Media deleting if marked.
          $has_media = $this->hasMedia($data['data'], $data['entity']->bundle());

          if (isset($has_media['queue_name']) && isset($has_media['origin'])) {
            $media_plugin = $this->queueManager->createInstance($has_media['queue_name']);
            $media_plugin->processMedia($data['data'], ['type' => $data['entity']->bundle()], NULL, $data['entity']->label(), $data['entity']);
          }
        }
      }
      else {
        // Notify about dry run.
        if ($this->config->get('dry_run')) {
          $status = $this->t('Running in Dry run mode therefore @entity_type entity @type "@title" was not deleted, but only marked for deleting because it is not attached to its parent anymore and has no other parents within Drupal.', $status_params);
          // And notify (in shell only).
          $this->notify($data['entity'], 'Deleted', $status);
        }
      }
    }
  }

}
